@@ -19,19 +19,26 @@ from __future__ import annotations
 
 import json
 import pickle
+from datetime import datetime
+from typing import TYPE_CHECKING
 from unittest import mock
 
 import httpx
 import pytest
 import uuid6
 from task_sdk import make_client, make_client_w_dry_run, make_client_w_responses
+from uuid6 import uuid7
 
-from airflow.sdk.api.client import RemoteValidationError, ServerResponseError
+from airflow.sdk import timezone
+from airflow.sdk.api.client import Client, RemoteValidationError, ServerResponseError
 from airflow.sdk.api.datamodels._generated import (
+    AssetEventsResponse,
     AssetResponse,
     ConnectionResponse,
     DagRunState,
     DagRunStateResponse,
+    HITLDetailResponse,
+    HITLUser,
     VariableResponse,
     XComResponse,
 )
@@ -39,13 +46,17 @@ from airflow.sdk.exceptions import ErrorType
 from airflow.sdk.execution_time.comms import (
     DeferTask,
     ErrorResponse,
+    HITLDetailRequestResult,
     OKResponse,
+    PreviousDagRunResult,
     RescheduleTask,
     TaskRescheduleStartDate,
 )
-from airflow.utils import timezone
 from airflow.utils.state import TerminalTIState
 
+if TYPE_CHECKING:
+    from time_machine import TimeMachineFixture
+
 
 class TestClient:
     @pytest.mark.parametrize(
@@ -78,6 +89,33 @@ class TestClient:
         assert resp.status_code == 200
         assert resp.json() == json_response
 
+    @mock.patch("airflow.sdk.api.client.API_SSL_CERT_PATH", "/capath/does/not/exist/")
+    def test_add_capath(self):
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            return httpx.Response(status_code=200)
+
+        with pytest.raises(FileNotFoundError) as err:
+            make_client(httpx.MockTransport(handle_request))
+
+        assert isinstance(err.value, FileNotFoundError)
+
+    @mock.patch("airflow.sdk.api.client.API_TIMEOUT", 60.0)
+    def test_timeout_configuration(self):
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            return httpx.Response(status_code=200)
+
+        client = make_client(httpx.MockTransport(handle_request))
+        assert client.timeout == httpx.Timeout(60.0)
+
+    def test_timeout_can_be_overridden(self):
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            return httpx.Response(status_code=200)
+
+        client = Client(
+            base_url="test://server", token="", transport=httpx.MockTransport(handle_request), timeout=120.0
+        )
+        assert client.timeout == httpx.Timeout(120.0)
+
     def test_error_parsing(self):
         responses = [
             httpx.Response(422, json={"detail": [{"loc": ["#0"], "msg": "err", "type": "required"}]})
@@ -875,6 +913,52 @@ class TestConnectionOperations:
         assert result.error == ErrorType.CONNECTION_NOT_FOUND
 
 
+class TestAssetEventOperations:
+    @pytest.mark.parametrize(
+        "request_params",
+        [
+            ({"name": "this_asset", "uri": "s3://bucket/key"}),
+            ({"alias_name": "this_asset_alias"}),
+        ],
+    )
+    def test_by_name_get_success(self, request_params):
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            params = request.url.params
+            if request.url.path == "/asset-events/by-asset":
+                assert params.get("name") == request_params.get("name")
+                assert params.get("uri") == request_params.get("uri")
+            elif request.url.path == "/asset-events/by-asset-alias":
+                assert params.get("name") == request_params.get("alias_name")
+            else:
+                return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+
+            return httpx.Response(
+                status_code=200,
+                json={
+                    "asset_events": [
+                        {
+                            "id": 1,
+                            "asset": {
+                                "name": "this_asset",
+                                "uri": "s3://bucket/key",
+                                "group": "asset",
+                            },
+                            "created_dagruns": [],
+                            "timestamp": "2023-01-01T00:00:00Z",
+                        }
+                    ]
+                },
+            )
+
+        client = make_client(transport=httpx.MockTransport(handle_request))
+        result = client.asset_events.get(**request_params)
+
+        assert isinstance(result, AssetEventsResponse)
+        assert len(result.asset_events) == 1
+        assert result.asset_events[0].asset.name == "this_asset"
+        assert result.asset_events[0].asset.uri == "s3://bucket/key"
+
+
 class TestAssetOperations:
     @pytest.mark.parametrize(
         "request_params",
@@ -963,7 +1047,7 @@ class TestDagRunOperations:
                     json={
                         "detail": {
                             "reason": "already_exists",
-                            "message": "A DAG Run already exists for DAG test_trigger_conflict with run id test_run_id",
+                            "message": "A Dag Run already exists for Dag test_trigger_conflict with run id test_run_id",
                         }
                     },
                 )
@@ -984,7 +1068,7 @@ class TestDagRunOperations:
                     json={
                         "detail": {
                             "reason": "already_exists",
-                            "message": "A DAG Run already exists for DAG test_trigger_conflict with run id test_run_id",
+                            "message": "A Dag Run already exists for Dag test_trigger_conflict with run id test_run_id",
                         }
                     },
                 )
@@ -1082,6 +1166,86 @@ class TestDagRunOperations:
         result = client.dag_runs.get_count(dag_id="test_dag", run_ids=["run1", "run2"])
         assert result.count == 2
 
+    def test_get_previous_basic(self):
+        """Test basic get_previous functionality with dag_id and logical_date."""
+        logical_date = datetime(2024, 1, 15, 12, 0, 0, tzinfo=timezone.utc)
+
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            if request.url.path == "/dag-runs/test_dag/previous":
+                assert request.url.params["logical_date"] == logical_date.isoformat()
+                # Return complete DagRun data
+                return httpx.Response(
+                    status_code=200,
+                    json={
+                        "dag_id": "test_dag",
+                        "run_id": "prev_run",
+                        "logical_date": "2024-01-14T12:00:00+00:00",
+                        "start_date": "2024-01-14T12:05:00+00:00",
+                        "run_after": "2024-01-14T12:00:00+00:00",
+                        "run_type": "scheduled",
+                        "state": "success",
+                        "consumed_asset_events": [],
+                    },
+                )
+            return httpx.Response(status_code=422)
+
+        client = make_client(transport=httpx.MockTransport(handle_request))
+        result = client.dag_runs.get_previous(dag_id="test_dag", logical_date=logical_date)
+
+        assert isinstance(result, PreviousDagRunResult)
+        assert result.dag_run.dag_id == "test_dag"
+        assert result.dag_run.run_id == "prev_run"
+        assert result.dag_run.state == "success"
+
+    def test_get_previous_with_state_filter(self):
+        """Test get_previous functionality with state filtering."""
+        logical_date = datetime(2024, 1, 15, 12, 0, 0, tzinfo=timezone.utc)
+
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            if request.url.path == "/dag-runs/test_dag/previous":
+                assert request.url.params["logical_date"] == logical_date.isoformat()
+                assert request.url.params["state"] == "success"
+                # Return complete DagRun data
+                return httpx.Response(
+                    status_code=200,
+                    json={
+                        "dag_id": "test_dag",
+                        "run_id": "prev_success_run",
+                        "logical_date": "2024-01-14T12:00:00+00:00",
+                        "start_date": "2024-01-14T12:05:00+00:00",
+                        "run_after": "2024-01-14T12:00:00+00:00",
+                        "run_type": "scheduled",
+                        "state": "success",
+                        "consumed_asset_events": [],
+                    },
+                )
+            return httpx.Response(status_code=422)
+
+        client = make_client(transport=httpx.MockTransport(handle_request))
+        result = client.dag_runs.get_previous(dag_id="test_dag", logical_date=logical_date, state="success")
+
+        assert isinstance(result, PreviousDagRunResult)
+        assert result.dag_run.dag_id == "test_dag"
+        assert result.dag_run.run_id == "prev_success_run"
+        assert result.dag_run.state == "success"
+
+    def test_get_previous_not_found(self):
+        """Test get_previous when no previous Dag run exists returns None."""
+        logical_date = datetime(2024, 1, 15, 12, 0, 0, tzinfo=timezone.utc)
+
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            if request.url.path == "/dag-runs/test_dag/previous":
+                assert request.url.params["logical_date"] == logical_date.isoformat()
+                # Return None (null) when no previous Dag run found
+                return httpx.Response(status_code=200, content="null")
+            return httpx.Response(status_code=422)
+
+        client = make_client(transport=httpx.MockTransport(handle_request))
+        result = client.dag_runs.get_previous(dag_id="test_dag", logical_date=logical_date)
+
+        assert isinstance(result, PreviousDagRunResult)
+        assert result.dag_run is None
+
 
 class TestTaskRescheduleOperations:
     def test_get_start_date(self):
@@ -1103,3 +1267,102 @@ class TestTaskRescheduleOperations:
 
         assert isinstance(result, TaskRescheduleStartDate)
         assert result.start_date == "2024-01-01T00:00:00Z"
+
+
+class TestHITLOperations:
+    def test_add_response(self) -> None:
+        ti_id = uuid7()
+
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            if request.url.path in (f"/hitlDetails/{ti_id}"):
+                return httpx.Response(
+                    status_code=201,
+                    json={
+                        "ti_id": str(ti_id),
+                        "options": ["Approval", "Reject"],
+                        "subject": "This is subject",
+                        "body": "This is body",
+                        "defaults": ["Approval"],
+                        "params": None,
+                        "multiple": False,
+                    },
+                )
+            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+
+        client = make_client(transport=httpx.MockTransport(handle_request))
+        result = client.hitl.add_response(
+            ti_id=ti_id,
+            options=["Approval", "Reject"],
+            subject="This is subject",
+            body="This is body",
+            defaults=["Approval"],
+            params=None,
+            multiple=False,
+        )
+        assert isinstance(result, HITLDetailRequestResult)
+        assert result.ti_id == ti_id
+        assert result.options == ["Approval", "Reject"]
+        assert result.subject == "This is subject"
+        assert result.body == "This is body"
+        assert result.defaults == ["Approval"]
+        assert result.params is None
+        assert result.multiple is False
+        assert result.assigned_users is None
+
+    def test_update_response(self, time_machine: TimeMachineFixture) -> None:
+        time_machine.move_to(datetime(2025, 7, 3, 0, 0, 0))
+        ti_id = uuid7()
+
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            if request.url.path in (f"/hitlDetails/{ti_id}"):
+                return httpx.Response(
+                    status_code=200,
+                    json={
+                        "chosen_options": ["Approval"],
+                        "params_input": {},
+                        "responded_by_user": {"id": "admin", "name": "admin"},
+                        "response_received": True,
+                        "responded_at": "2025-07-03T00:00:00Z",
+                    },
+                )
+            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+
+        client = make_client(transport=httpx.MockTransport(handle_request))
+        result = client.hitl.update_response(
+            ti_id=ti_id,
+            chosen_options=["Approve"],
+            params_input={},
+        )
+        assert isinstance(result, HITLDetailResponse)
+        assert result.response_received is True
+        assert result.chosen_options == ["Approval"]
+        assert result.params_input == {}
+        assert result.responded_by_user == HITLUser(id="admin", name="admin")
+        assert result.responded_at == timezone.datetime(2025, 7, 3, 0, 0, 0)
+
+    def test_get_detail_response(self, time_machine: TimeMachineFixture) -> None:
+        time_machine.move_to(datetime(2025, 7, 3, 0, 0, 0))
+        ti_id = uuid7()
+
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            if request.url.path in (f"/hitlDetails/{ti_id}"):
+                return httpx.Response(
+                    status_code=200,
+                    json={
+                        "chosen_options": ["Approval"],
+                        "params_input": {},
+                        "responded_by_user": {"id": "admin", "name": "admin"},
+                        "response_received": True,
+                        "responded_at": "2025-07-03T00:00:00Z",
+                    },
+                )
+            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+
+        client = make_client(transport=httpx.MockTransport(handle_request))
+        result = client.hitl.get_detail_response(ti_id=ti_id)
+        assert isinstance(result, HITLDetailResponse)
+        assert result.response_received is True
+        assert result.chosen_options == ["Approval"]
+        assert result.params_input == {}
+        assert result.responded_by_user == HITLUser(id="admin", name="admin")
+        assert result.responded_at == timezone.datetime(2025, 7, 3, 0, 0, 0)
